@@ -11,13 +11,10 @@ from oasys.widgets import congruence
 from oasys.util.oasys_util import TriggerIn, EmittingStream
 
 from orangecontrib.wofry.util.wofry_objects import WofryData
-from orangecontrib.wofry.widgets.gui.ow_wofry_widget import WofryWidget
-from orangecontrib.xoppy.util.python_script import PythonScript  # TODO: change import from wofry!!!
+from orangecontrib.esrf.wofry.widgets.gui.ow_wofry_widget import WofryWidget # TODO: from orangecontrib.wofry.widgets.gui.ow_wofry_widget import WofryWidget
 
 from syned.widget.widget_decorator import WidgetDecorator
 
-from wofry.propagator.wavefront1D.generic_wavefront import GenericWavefront1D
-
 import xraylib
 
 import os
@@ -49,7 +46,7 @@ class OWRefractiveCorrector1D(WofryWidget):
                 "id":"Trigger"}]
 
     inputs = [("WofryData", WofryData, "set_input"),
-              ("GenericWavefront1D", GenericWavefront1D, "set_input"),
+              # ("GenericWavefront1D", GenericWavefront1D, "set_input"),
               WidgetDecorator.syned_input_data()[0]]
 
     correction_method = Setting(1)
@@ -71,17 +68,7 @@ class OWRefractiveCorrector1D(WofryWidget):
     titles = ["Wavefront (input) 1D Intensity", "Wavefront (input) 1D Phase", "Wavefront (target) 1D Phase", "Correction Profile"]
 
     def __init__(self):
-        super().__init__(is_automatic=True, show_view_options=True)
-
-        #
-        # add script tab to tabs panel
-        #
-        script_tab = oasysgui.createTabPage(self.main_tabs, "Script")
-        self.wofry_script = PythonScript()
-        self.wofry_script.code_area.setFixedHeight(400)
-        script_box = gui.widgetBox(script_tab, "Python script", addSpace=True, orientation="horizontal")
-        script_box.layout().addWidget(self.wofry_script)
-
+        super().__init__(is_automatic=True, show_view_options=True, show_script_tab=True)
 
         #
         # build control panel
@@ -112,7 +99,7 @@ class OWRefractiveCorrector1D(WofryWidget):
 
 
         gui.comboBox(box_corrector, self, "correction_method", label="Correction type", labelWidth=350,
-                     items=["None","Focus to waist with reflector"],
+                     items=["None","Focus to waist"],
                      callback=self.set_visible,
                      sendSelectedValue=False, orientation="horizontal")
 
